/*-------------------------------------------------------------------------
 * synrebuffer.c - synrebuffer
 *-------------------------------------------------------------------------
 */
#include <syn.h>
#include <strings.h>
#include <event.h>

/*------------------------------------------------------------------------
 * synrebuffer - rebuffer a synchronization stream
 *------------------------------------------------------------------------
 */
int
synrebuffer(struct synsession *pssn, ssrc_t ssrc)
{

	struct synstream *psstm;
	struct session *psn;

	psn = pssn->ssn_session;
	psstm = syngetstream(pssn, ssrc);
	if (psstm == NULL)
		return ERROR;

	if (pthread_mutex_lock(&psstm->sstm_readmutex) != 0) {
		synreleasestream(pssn, psstm);
		return ERROR;
	}

	/*
	 * Call rtpstream off to clear the queue.
	 */
	rtpstreamoff(psn, psstm->sstm_ssrc);
  
	psstm->sstm_buffering = TRUE;
	psstm->sstm_lastread = 0;
	psstm->sstm_clky = 0;
	memset(&psstm->sstm_clkx, 0, sizeof(psstm->sstm_clkx));
  
	/*
	 * During rebuffing need event for each RTP packet received
	 * so that synnewdata is invoked to check whether each
	 * packet received for the stream completes buffering.
	 */
	eventqfilterssrc(&pssn->ssn_events, EVENT_RTP, psstm->sstm_ssrc, TRUE);
  
	/*
	 * Wake up any reading threads so that they know the stream 
	 * is buffering.
	 */
	pthread_cond_broadcast(&psstm->sstm_readcond);
	pthread_mutex_unlock(&psstm->sstm_readmutex);

	/*
	 * Turn the stream back on at the RTP layer.
	 */
	rtpstreamon(psn, psstm->sstm_ssrc, psstm->sstm_clkrt);
	synreleasestream(pssn, psstm);
	return OK;
}
