/*-------------------------------------------------------------------------
 * payload.c - encodings, getencoding
 *-------------------------------------------------------------------------
 */

#include <payload.h>

/*
 * -1 == var, N/A, or unknown
 * Chan refers to the number of channels the rtplib
 * must demultiplex. For instance, MPEG audio (MPA) will 
 * always have one multiplexed channel because MPEG encodes
 * stereo internally within each frame rather than alternating
 * left/right channel frames.
 */

struct encoding encodings[] = {
  /*ID, NAME, TYPE (A/V), Format,     CLKRT, Chan, UnitSz, ms/Audio Frame */
  {0,   "PCMU", ENC_AUDIO, ENC_SAMPLE, 8000,   1,   8,  -1},
  {1,   "1016", ENC_AUDIO, ENC_FRAME,  8000,   1,  -1,   240},
  {2,   "G721", ENC_AUDIO, ENC_SAMPLE, 8000,   1,   4,  -1},
  {3,   "GSM",  ENC_AUDIO, ENC_FRAME,  8000,   1,  -1,   160},
  {4,   "G723", ENC_AUDIO, ENC_FRAME,  8000,   1,  -1,  -1},
  {5,   "DVI4", ENC_AUDIO, ENC_SAMPLE, 8000,   1,   4,  -1},
  {6,   "DVI4", ENC_AUDIO, ENC_SAMPLE, 16000,  1,   4,  -1},
  {7,   "LPC",  ENC_AUDIO, ENC_FRAME,  8000,   1,  -1,   160},
  {8,   "PCMA", ENC_AUDIO, ENC_SAMPLE, 8000,   1,   8,  -1},
  {9,   "G722", ENC_AUDIO, ENC_SAMPLE, 8000,   1,   8,  -1},
  {10,  "L16",  ENC_AUDIO, ENC_SAMPLE, 44100,  2,   16, -1},
  {11,  "L16",  ENC_AUDIO, ENC_SAMPLE, 44100,  1,   16, -1},
  {14,  "MPA",  ENC_AUDIO, ENC_FRAME,  90000,  1,  -1,  -1},
  {15,  "G728", ENC_AUDIO, ENC_FRAME,  8000,   1,  -1,  -1},
  {25,  "CelB", ENC_VIDEO, ENC_FRAME,  90000, -1,  -1,  -1},
  {26,  "JPEG", ENC_VIDEO, ENC_FRAME,  90000, -1,  -1,  -1,},
  {28,  "nv",   ENC_VIDEO, ENC_FRAME,  90000, -1,  -1,  -1}, 
  {31,  "H261", ENC_VIDEO, ENC_FRAME,  90000, -1,  -1,  -1}, 
  {32,  "MPV",  ENC_VIDEO, ENC_FRAME,  90000, -1,  -1,  -1},
  {33,  "MP2T", ENC_AV,    ENC_FRAME,  90000, -1,  -1,-  1},
  {122, "L16",  ENC_AUDIO, ENC_SAMPLE, 8000,   1,  16,-  1}, /* RAT */
  {-1,  "UNKN", -1, 	   -1,	       -1,    -1,  -1,  -1}};

/*------------------------------------------------------------------------
 * getencoding - lookup and return a struct encoding by payload type
 *------------------------------------------------------------------------
 */
struct encoding
getencoding(int pt)
{
	int i;

	for (i = 0; encodings[i].en_pt != -1; i++) {
		if (encodings[i].en_pt == pt)
			break;
	}
	return encodings[i];
}
